@@ -108,7 +108,6 @@ def _validate_ref(data):
 
 _validate_code = _validate_string
 
-# still not sure what is actually stored here, but i know how big it is...
 def _validate_code_w_scope(data):
     (length, data) = _get_int(data)
     assert len(data) >= length + 1
@@ -217,6 +216,12 @@ def _get_date(data):
     seconds = float(struct.unpack("<q", data[:8])[0]) / 1000.0
     return (datetime.datetime.utcfromtimestamp(seconds), data[8:])
 
+def _get_code_w_scope(data):
+    (_, data) = _get_int(data)
+    (code, data) = _get_string(data)
+    (scope, data) = _get_object(data)
+    return (Code(code, scope), data)
+
 def _get_null(data):
     return (None, data)
 
@@ -258,7 +263,7 @@ _element_getter = {
     "\x0C": _get_ref,
     "\x0D": _get_string, # code
     "\x0E": _get_string, # symbol
-#    "\x0F": _get_code_w_scope
+    "\x0F": _get_code_w_scope,
     "\x10": _get_int, # number_int
 }
 
@@ -291,14 +296,16 @@ def _element_to_bson(key, value):
     if isinstance(value, float):
         return "\x01" + name + struct.pack("<d", value)
     if isinstance(value, Binary):
-        subtype = value.subtype()
+        subtype = value.subtype
         if subtype == 2:
             value = struct.pack("<i", len(value)) + value
         return "\x05" + name + struct.pack("<i", len(value)) + chr(subtype) + value
     if isinstance(value, Code):
         cstring = _make_c_string(value)
+        scope = _dict_to_bson(value.scope)
+        full_length = struct.pack("<i", 8 + len(cstring) + len(scope))
         length = struct.pack("<i", len(cstring))
-        return "\x0D" + name + length + cstring
+        return "\x0F" + name + full_length + length + cstring + scope
     if isinstance(value, str):
         cstring = _make_c_string(value)
         length = struct.pack("<i", len(cstring))
@@ -342,8 +349,8 @@ def _element_to_bson(key, value):
             flags += "x"
         return "\x0B" + name + _make_c_string(pattern) + _make_c_string(flags)
     if isinstance(value, DBRef):
-        ns = _make_c_string(value.collection())
-        return "\x0C" + name + struct.pack("<i", len(ns)) + ns + _shuffle_oid(str(value.id()))
+        ns = _make_c_string(value.collection)
+        return "\x0C" + name + struct.pack("<i", len(ns)) + ns + _shuffle_oid(str(value.id))
     raise InvalidDocument("cannot convert value of type %s to bson" % type(value))
 
 def _dict_to_bson(dict):
