@@ -24,7 +24,7 @@ except: # for Python < 2.5
 
 from son import SON
 from dbref import DBRef
-from son_manipulator import ObjectIdInjector, ObjectIdShuffler
+from son_manipulator import ObjectIdInjector, ObjectIdShuffler, DBRefTransformer
 from collection import Collection
 from errors import InvalidName, CollectionInvalid, OperationFailure
 from code import Code
@@ -49,7 +49,11 @@ class Database(object):
 
         self.__name = unicode(name)
         self.__connection = connection
-        self.__manipulators = [ObjectIdInjector(self), ObjectIdShuffler(self)]
+        self.__manipulators = []
+        self.__copying_manipulators = []
+        self.add_son_manipulator(ObjectIdInjector())
+        self.add_son_manipulator(ObjectIdShuffler())
+        self.add_son_manipulator(DBRefTransformer())
 
     def __check_name(self, name):
         for invalid_char in [" ", ".", "$", "/", "\\"]:
@@ -58,15 +62,18 @@ class Database(object):
         if not name:
             raise InvalidName("database name cannot be the empty string")
 
-    def add_son_manipulator(self, manipulator, index=-1):
+    def add_son_manipulator(self, manipulator):
         """Add a new son manipulator to this database.
 
+        Newly added manipulators will be applied before existing ones.
+
         :Parameters:
           - `manipulator`: the manipulator to add
-          - `index` (optional): the index to add the manipulator on the
-            manipulator list
         """
-        self.__manipulators[index:index] = [manipulator]
+        if manipulator.will_copy():
+            self.__copying_manipulators.insert(0, manipulator)
+        else:
+            self.__manipulators.insert(0, manipulator)
 
     def connection(self):
         """Get the database connection.
@@ -139,6 +146,8 @@ class Database(object):
         """
         for manipulator in self.__manipulators:
             son = manipulator.transform_incoming(son, collection)
+        for manipulator in self.__copying_manipulators:
+            son = manipulator.transform_incoming(son, collection)
         return son
 
     def _fix_outgoing(self, son, collection):
@@ -148,7 +157,9 @@ class Database(object):
           - `son`: the son object coming out of the database
           - `collection`: the collection the son object was saved in
         """
-        for manipulator in self.__manipulators:
+        for manipulator in reversed(self.__manipulators):
+            son = manipulator.transform_outgoing(son, collection)
+        for manipulator in reversed(self.__copying_manipulators):
             son = manipulator.transform_outgoing(son, collection)
         return son
 
@@ -350,7 +361,7 @@ class Database(object):
         """
         if not isinstance(dbref, DBRef):
             raise TypeError("cannot dereference a %s" % type(dbref))
-        return self[dbref.collection()].find_one(dbref.id())
+        return self[dbref.collection].find_one({"_id": dbref.id})
 
     def eval(self, code, *args):
         """Evaluate a JavaScript expression on the Mongo server.
@@ -361,8 +372,8 @@ class Database(object):
         arguments will be passed to that function when it is run on the
         server.
 
-        Raises TypeError if `code` is not an instance of (str, unicode). Raises
-        OperationFailure if the eval fails. Returns the result of the
+        Raises TypeError if `code` is not an instance of (str, unicode, `Code`).
+        Raises OperationFailure if the eval fails. Returns the result of the
         evaluation.
 
         :Parameters:
@@ -370,9 +381,9 @@ class Database(object):
           - `args` (optional): additional positional arguments are passed to
             the `code` being evaluated
         """
-        if not isinstance(code, types.StringTypes):
-            raise TypeError("code must be an instance of (str, unicode)")
+        if not isinstance(code, Code):
+            code = Code(code)
 
-        command = SON([("$eval", Code(code)), ("args", list(args))])
+        command = SON([("$eval", code), ("args", list(args))])
         result = self._command(command)
         return result.get("retval", None)
