@@ -21,8 +21,11 @@ sys.path[0:0] = [""]
 
 import qcheck
 from pymongo.objectid import ObjectId
+from pymongo.dbref import DBRef
 from pymongo.son import SON
-from pymongo.son_manipulator import SONManipulator, ObjectIdInjector, NamespaceInjector, ObjectIdShuffler
+from pymongo.son_manipulator import SONManipulator, ObjectIdInjector
+from pymongo.son_manipulator import NamespaceInjector, ObjectIdShuffler
+from pymongo.son_manipulator import DBRefTransformer
 from pymongo.database import Database
 from test_connection import get_connection
 
@@ -31,7 +34,7 @@ class TestSONManipulator(unittest.TestCase):
         self.db = Database(get_connection(), "pymongo_test")
 
     def test_basic(self):
-        manip = SONManipulator(self.db)
+        manip = SONManipulator()
         collection = self.db.test
 
         def incoming_is_identity(son):
@@ -45,7 +48,7 @@ class TestSONManipulator(unittest.TestCase):
                               qcheck.gen_mongo_dict(3))
 
     def test_id_injection(self):
-        manip = ObjectIdInjector(self.db)
+        manip = ObjectIdInjector()
         collection = self.db.test
 
         def incoming_adds_id(son):
@@ -61,7 +64,7 @@ class TestSONManipulator(unittest.TestCase):
                               qcheck.gen_mongo_dict(3))
 
     def test_id_shuffling(self):
-        manip = ObjectIdShuffler(self.db)
+        manip = ObjectIdShuffler()
         collection = self.db.test
 
         def incoming_moves_id(son_in):
@@ -85,7 +88,7 @@ class TestSONManipulator(unittest.TestCase):
                               qcheck.gen_mongo_dict(3))
 
     def test_ns_injection(self):
-        manip = NamespaceInjector(self.db)
+        manip = NamespaceInjector()
         collection = self.db.test
 
         def incoming_adds_ns(son):
@@ -100,5 +103,17 @@ class TestSONManipulator(unittest.TestCase):
         qcheck.check_unittest(self, outgoing_is_identity,
                               qcheck.gen_mongo_dict(3))
 
+    def test_dbref_transformer(self):
+        manip = DBRefTransformer()
+        collection = self.db.test
+
+        self.assertEqual({}, manip.transform_incoming({}, collection))
+        self.assertEqual({"foo": {"$ref": "test", "$id": 5}},
+                         manip.transform_incoming({"foo": DBRef("test", 5)}, collection))
+
+        self.assertEqual({}, manip.transform_outgoing({}, collection))
+        self.assertEqual({"foo": DBRef("test", 5)},
+                         manip.transform_outgoing({"foo": {"$ref": "test", "$id": 5}}, collection))
+
 if __name__ == "__main__":
     unittest.main()
